import Fastify from "fastify";
import { Yapper } from "./Yapper";
import { state } from "./utils";
import { reqSpotify, spotifyFilter } from "./spotify";

const fastify = Fastify({
	loggerInstance: new Yapper(),
	disableRequestLogging: true
});

fastify.get("/", (request, reply) => {
	reply.type("text/html");
	return '<a href="/auth">auth</a>';
});

fastify.get("/auth", async (request, reply) => {
	return reply.redirect(
		`https://accounts.spotify.com/authorize?` +
			new URLSearchParams({
				response_type: "code",
				client_id: process.env.CLIENT_ID!,
				scope: "user-library-read user-library-modify playlist-read-private playlist-read-collaborative playlist-modify-private playlist-modify-public",
				redirect_uri: `https://${request.hostname}/callback`,
				show_dialog: "true",
				state: "ballsgamingmeoww"
			}).toString()
	);
});

fastify.get("/callback", async (request, reply) => {
	if ((request.query as any).error) return reply.code(400).send();

	const tokenRes = await (
		await fetch("https://accounts.spotify.com/api/token", {
			method: "POST",
			headers: {
				"Content-Type": "application/x-www-form-urlencoded",
				Authorization:
					"Basic " +
					Buffer.from(
						process.env.CLIENT_ID + ":" + process.env.CLIENT_SECRET
					).toString("base64")
			},
			body: new URLSearchParams({
				code: (request.query as any).code,
				redirect_uri: `https://${request.hostname}/callback`,
				grant_type: "authorization_code"
			})
		})
	).json();

	if (tokenRes.error) return reply.code(400).send();

	state.accessToken = tokenRes.access_token;
	state.explodesAt = Date.now() + tokenRes.expires_in * 1000;
	state.refreshToken = tokenRes.refresh_token;

	return `Logged in as ${
		(await reqSpotify("/me")).display_name
	}. You're good to go!`;
});

try {
	setInterval(spotifyFilter, 5000);
	spotifyFilter();

	fastify.listen({
		port: 4929,
		host: "0.0.0.0"
	});
} catch (err) {
	fastify.log.error(err);
	process.exit(1);
}
