import { state } from "./utils";

export async function reqSpotify(
	endpoint: string,
	method: string = "GET",
	data?: object
) {
	if (state.explodesAt <= Date.now()) {
		console.log("We will be refreshing the token");
		const tokenRes = await (
			await fetch("https://accounts.spotify.com/api/token", {
				method: "POST",
				headers: {
					"Content-Type": "application/x-www-form-urlencoded",
					Authorization: `Basic ${Buffer.from(
						process.env.CLIENT_ID + ":" + process.env.CLIENT_SECRET
					).toString("base64")}`
				},
				body: new URLSearchParams({
					refresh_token: state.refreshToken,
					grant_type: "refresh_token"
				})
			})
		).json();
		console.log(tokenRes);

		if (!tokenRes.error) {
			state.accessToken = tokenRes.access_token;
			state.explodesAt = Date.now() + tokenRes.expires_in * 1000;
			state.refreshToken = tokenRes.refresh_token || state.refreshToken;
		}
	}

	try {
		return await (
			await fetch("https://api.spotify.com/v1" + endpoint, {
				method,
				headers: {
					Authorization: `Bearer ${state.accessToken}`
				},
				body: JSON.stringify(data)
			})
		).json();
	} catch {
		return {};
	}
}

export async function spotifyFilter() {
	const likedSongs = (await reqSpotify("/me/tracks")).items.map((item) => {
		return {
			uri: item.track.uri
		};
	});

	await reqSpotify("/playlists/0jWbMtGfptAzenepLTL4ED/tracks", "DELETE", {
		tracks: likedSongs
	});
	await reqSpotify("/playlists/0jWbMtGfptAzenepLTL4ED/tracks", "POST", {
		uris: likedSongs.map((lss) => lss.uri),
		position: 0
	});
	await reqSpotify("/me/tracks", "DELETE", {
		ids: likedSongs.map((lss) => lss.uri.replace("spotify:track:", ""))
	});
}
